@@ -1,8 +1,14 @@
 """Messaging library for Python"""
 from __future__ import absolute_import
 
-VERSION = (3, 0, 5)
-__version__ = '.'.join(map(str, VERSION[0:3])) + ''.join(VERSION[3:])
+from collections import namedtuple
+
+version_info_t = namedtuple(
+    'version_info_t', ('major', 'minor', 'micro', 'releaselevel', 'serial'),
+)
+
+VERSION = version_info_t(3, 0, 6, '', '')
+__version__ = '{0.major}.{0.minor}.{0.micro}{0.releaselevel}'.format(VERSION)
 __author__ = 'Ask Solem'
 __contact__ = 'ask@celeryproject.org'
 __homepage__ = 'http://kombu.readthedocs.org'
